local L = LibStub("AceLocale-3.0"):GetLocale("Broker_AddonControl")

Broker_AddonControlConfig = {}
Broker_AddonControlChar = {}
Broker_AddonControl = {}

local f = CreateFrame("Frame","Broker_AddonControlFrame", UIParent) -- main frame

local slider, UpdateTablet
local sliderValue, hasSlider = 0
local tip = true
local RegisteredEvents, DefaultConfig, buttons = {}
sortedSets ,groupSets = {}
local toolTipModifier = ""
local queuedForDelete = ""

local MIN_DESC_WIDTH = 140
local	BUTTON_HEIGHT,	ICON_SIZE,	GAP,	TEXT_OFFSET,	MAX_ENTRIES =
	15,		15,		5,	5

f:SetScript("OnEvent", function (self, event, ...) if (RegisteredEvents[event]) then return RegisteredEvents[event](self, event, ...) end end)

local colors = {
	background = { .1, .1, .1, .85 },
	border = { .3, .3, .3, .9 },
	allEnabled = { .1, .8, .1 },
	nonEnabled = { 1, .1, .1 },	
	partEnabled = { .1, .1, .9 },	
	addonGroup = { .8, .6, .2 },	
	activeAddon = { .3, .8, .3 },	
	enabledAddon = { 1, .8, .4 },	
	disabledAddon = { .5, .5, .5 },	
	loadedAddon = { .2, .8, .2 },		
	clickToLoad = { .05, .95, .05 },	
	altToolTip = { 1, 1, 1 },	
}

DefaultConfig = {
	ignoreHardCodedGroups = false,
	altReloadUI = false,
	showBlockHints = true,
	scale = 1,
	maxMatchCharacters = 4,
	maxDescWidth = 200,	
	resizeTip = true,
	resizeNotes = true,
	showAddonTipInfo = true,
	hideNotes = false,
}

local highlight = f:CreateTexture()
highlight:SetTexture"Interface\\QuestFrame\\UI-QuestTitleHighlight"
highlight:SetBlendMode"ADD"
highlight:SetAlpha(0)

local function reclaim(t)
	if type(t) ~= "table" then return end
	for k, v in pairs(t) do
		if type(v) == "table" then
			t[k] = reclaim(v)
		else
			t[k] = nil
		end
	end
	t = nil
	return nil
end

local function UpdateBlockHints()
	if f.onBlock then
		if Broker_AddonControl.GetConfigValue("showBlockHints") then
			tip:SetOwner(f, "ANCHOR_NONE")
			tip:SetPoint( f.isTop and "TOP" or "BOTTOM", f, f.isTop and "BOTTOM" or "TOP" )
			tip:AddLine(L["Hints"].." [|cffffffffBlock|r]")
			--tip:AddLine("|cffff8020Click|r to open panel.", 0.2, 1, 0.2) 
			tip:AddLine("|cffff8020"..L["Click"].."|r "..L["to Reload UI."], 0.2, 1, 0.2) 
			tip:AddLine("|cffff8020"..L["Right"]..L["Click"].."|r "..L["to display config menu."], 0.2, 1, 0.2) 
			tip:AddLine("|cffff8020Shift+"..L["Click"].."|r "..L["to save this addon set."], .2, 1, .2)			
			tip:AddLine("|cffff8020"..L["Hold Alt+Mouseover"].."|r "..L["to unhide addons."], 0.2, 1, 0.2) 
			tip:AddLine("|cffff8020"..L["Hold Shift+Mouseover"].."|r "..L["to load/delete addon sets."], 0.2, 1, 0.2) 
			if tip:NumLines() > 1 then tip:Show() else tip:Hide() end
		else
			tip:Hide()
		end
	end
end

local function ShowHints(btn)
	if Broker_AddonControl.GetConfigValue("showBlockHints") and btn and btn.addonName then
		local showBelow = UIParent:GetHeight()/Broker_AddonControl.GetConfigValue("scale")-f:GetTop() < f:GetBottom()
		tip:SetOwner(f, "ANCHOR_NONE")
		tip:SetPoint(showBelow and "TOP" or "BOTTOM", f, showBelow and "BOTTOM" or "TOP")
		if (btn.isNode == 1) then -- its a node
			tip:AddLine(L["Hints"].." [|cffffffffGroup|r]")
			tip:AddLine("|cffff8020"..L["Click"].."|r "..L["to expand/collapse."], .2, 1, .2)
			tip:AddLine("|cffff8020Shift+"..L["Click"].."|r "..L["to enable/disable all addons in group."], .2, 1, .2)
			tip:AddLine("|cffff8020Ctrl+"..L["Click"].."|r "..L["to expand/collapse all nodes."], .2, 1, .2)
		else
			if (Broker_AddonControl.GetConfigValue("showAddonTipInfo")) and (toolTipModifier ~= "Shift") then
				tip:AddLine(btn.addonName);
				local info = GetAddOnMetadata(btn.addonName, "X-Category");
				if (info) then tip:AddLine("|cffffff20"..L["Category"].."|r "..info, 1, 1, 1) end
				info = GetAddOnMetadata(btn.addonName, "Author");
				if (info) then tip:AddLine("|cffffff20"..L["Author"].."|r "..info, 1, 1, 1) end				
				info = GetAddOnMetadata(btn.addonName, "Version");
				if (info) then tip:AddLine("|cffffff20"..L["Version"].."|r "..info, 1, 1, 1) end				
				info = GetAddOnMetadata(btn.addonName, "Notes");
				if (info) then tip:AddLine("|cffffff20"..L["Notes"].."|r "..info, 1, 1, 1,1) end								
				if (btn.enabled) then
					local mem = GetAddOnMemoryUsage(btn.addonName)
					local text2
					if mem > 1024 then text2 = ("|cff8080ff%.2f|r MiB"):format(mem / 1024)					
					else text2 = ("|cff8080ff%.0f|r KiB"):format(mem) end
					tip:AddLine("|cffffff20"..L["Memory"].."|r "..text2, 1, 1, 1)
				end
				tip:AddLine(" ", 1, 1, 1)
			end
			if (toolTipModifier == "Shift") then							
				tip:AddLine(L["Hints"].." [|cffffffff"..L["set mode"].."|r]")
				tip:AddLine("|cffff8020"..L["Click"].."|r "..L["to load the selected set."], .2, 1, .2)			
				tip:AddLine("|cffff8020Shift+"..L["Click"].."|r "..L["to load the selected set and Reload the UI."], .2, 1, .2)
				tip:AddLine("|cffff8020Alt+"..L["Click"].."|r "..L["to remove the selected set."], .2, 1, .2)				
			elseif (toolTipModifier == "Alt") then
				tip:AddLine(L["Hints"].." [|cffffffff"..L["hidden mode"].."|r]")
				tip:AddLine("|cffff8020"..L["Click"].."|r "..L["to show this addon again."], .2, 1, .2)
			else			
				tip:AddLine(L["Hints"])
				tip:AddLine("|cffff8020"..L["Click"].."|r "..L["to enable/disable."], .2, 1, .2)
				tip:AddLine("|cffff8020Alt+"..L["Click"].."|r "..L["to hide the addon."], .2, 1, .2)
			end
		end
		
		if Broker_AddonControl.GetConfigValue("resizeTip") then tip:AddLine("|cffff8020Ctrl+"..L["MouseWheel"].."|r "..L["to resize tooltip."], .2, 1, .2) end
		if (toolTipModifier == "") then
			if Broker_AddonControl.GetConfigValue("resizeNotes") then tip:AddLine("|cffff8020Shift+"..L["MouseWheel"].."|r "..L["to resize addon notes."], .2, 1, .2) end
		end
		if tip:NumLines() > 1 then tip:Show() end
	end
end

local function Menu_OnEnter(b)
	if b and b.index then		
		highlight:SetAllPoints(b)
		if b.index > 0 then
			highlight:SetAlpha(1)
			ShowHints(b)
		end
	end
end

local function Menu_OnLeave(b)
	highlight:ClearAllPoints()	
	tip:Hide()
	if b and b.index and b.index > 0 then highlight:SetAlpha(0) end
	if not f:IsMouseOver() then f:Hide() end
end

local function Block_OnLeave(self)
	f.onBlock = nil
	tip:Hide()
	if not f:IsMouseOver() then
		f:Hide()
	end
end

local function UpdateScrollButtons(nbEntries)
	for i=1, #buttons do buttons[i]:Hide() end
	local sliderValue = hasSlider and sliderValue or 0
	for i=1, nbEntries do
		local button = buttons[sliderValue+i]
		button:SetPoint( "TOPLEFT", f, "TOPLEFT", GAP, -GAP - ((i-1)*(BUTTON_HEIGHT)) )
		button:Show()
	end
end

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "Broker_AddonControl") then
		SLASH_BAC1 = '/bac'
		SlashCmdList["BAC"] = function (msg, editbox)
			Broker_AddonControl.SlashCmdHandler(msg, editbox)	
		end
		setmetatable(Broker_AddonControlConfig, {__index = DefaultConfig})
		--Broker_AddonControlUI.CreateConfigMenu()
		--Broker_AddonControl.SetConfigValue("selfCastOn",GetCVar("autoSelfCast"));			
		
		f:Hide()
		f:SetBackdrop( {
			bgFile = "Interface\\Buttons\\WHITE8X8",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			edgeSize=16, tile = false, tileSize=0,
			insets = { left=3, right=3, top=3, bottom=3 } } )
		f:SetFrameStrata"TOOLTIP"
		f:SetClampedToScreen(true)
		f:EnableMouse(true)		
		f:SetScript( "OnEnter", Menu_OnEnter )
		f:SetScript( "OnLeave", Menu_OnLeave )		
		
		slider = CreateFrame("Slider", nil, f)
		slider:SetWidth(16)
		slider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
		slider:SetBackdrop( {
			bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
			edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
			edgeSize = 8, tile = true, tileSize = 8,
			insets = { left=3, right=3, top=6, bottom=6 }
		} )
		slider:SetValueStep(1)
		slider:SetScript( "OnLeave", Menu_OnLeave )
		slider:SetScript( "OnValueChanged", function(self, value)
			if hasSlider then
				sliderValue = value
				if f:IsMouseOver() then UpdateScrollButtons(MAX_ENTRIES) end
			end
		end )		
		
		tip = GameTooltip
		
		if (not Broker_AddonControlChar.expandedNodes) then
			Broker_AddonControlChar.expandedNodes = {};
		end
		if (not Broker_AddonControlChar.hiddenAddons) then
			Broker_AddonControlChar.hiddenAddons = {};
		end
		if (not Broker_AddonControlConfig.sets) then
			Broker_AddonControlConfig.sets = {};
		end	
		
		Broker_AddonControl.VerifyHiddenAddonsDisabled()
		Broker_AddonControl.UpdateText();
		Broker_AddonControl.CreateAddonGroups();
		Broker_AddonControlUI.CreateConfigMenu();
		
		if not StaticPopupDialogs["BrokerAddonControlAddSet"] then
			StaticPopupDialogs["BrokerAddonControlAddSet"] = {
				text = L["Enter the name for your saved Addon Set here !"],
				button1 = L["Add"],
				button2 = L["Cancel"],
				OnCancel = function() Broker_AddonControl.CancelCreateSet() end,
				sound = "levelup2",
				whileDead = 1,
				hideOnEscape = 1,
				timeout = 0,
				hasEditBox = 1,
				OnShow = function(self,data)
					self.editBox:SetText("")
				end,
				OnHide = function(self,data)
					self.editBox:SetText("")
				end,
				EditBoxOnEnterPressed = function(self)
					local string name = self:GetText()
					Broker_AddonControl.CreateSet(name)
				end,
				EditBoxOnEscapePressed = function(self,data)
					Broker_AddonControl.CancelCreateSet()
				end,
				OnAccept = function(self,data)
					local string name = self.editBox:GetText()
					Broker_AddonControl.CreateSet(name)
				end,				
			}
		end
		
		if not StaticPopupDialogs["BrokerAddonControlDeleteSet"] then
			StaticPopupDialogs["BrokerAddonControlDeleteSet"] = {
				text = L["Are you sure you want to delete the set %s?"],
				button1 = L["Delete"],
				button2 = L["Cancel"],
				whileDead = 1,
				showAlert = 1,
				timeout = 0,
				OnAccept = function() Broker_AddonControl.DeleteSet(queuedForDelete, true) end,
				OnCancel = function() queuedForDelete = nil end,
			}
		end				
		print("Broker_AddonControl Loaded. Type /bsc for usage")		
	end	
end

for k, v in pairs(RegisteredEvents) do
	f:RegisterEvent(k)
end

local function CreateFS( parent, justify, anchor, offsetX, color )
	local fs = parent:CreateFontString( nil, "OVERLAY", "SystemFont_Shadow_Med1" )
	if justify then fs:SetJustifyH( justify ) end
	if anchor then fs:SetPoint( "LEFT", anchor, "RIGHT", offsetX or GAP, 0 ) end
	if color then fs:SetTextColor(unpack(color)) end
	return fs
end

local function CreateTex( parent, anchor, offsetX )
	local tex = parent:CreateTexture()
	tex:SetWidth( ICON_SIZE )
	tex:SetHeight( ICON_SIZE )
	tex:SetPoint( "LEFT", anchor or parent, anchor and "RIGHT" or "LEFT", offsetX or 0, 0 )
	return tex
end

local function OnAddonClick( self, button )
	if (toolTipModifier == "Shift") then -- set mode
		if (IsAltKeyDown()) then
			Broker_AddonControl.DeleteSet(self.addonName);
		else
			Broker_AddonControl.UseSet(self.addonName);
			if (IsShiftKeyDown()) then
				ReloadUI();	
			end
		end
	elseif (toolTipModifier == "Alt") then				
		Broker_AddonControlChar.hiddenAddons[self.addonName] = nil;
	else
		if (self.isNode == 0) then -- its an addon
			if (IsAltKeyDown()) then
				if (not (self.master == 1)) then
					Broker_AddonControlChar.hiddenAddons[self.addonName] = 1;
					DisableAddOn(self.addonName);
				end
				UpdateTablet();
				return;
			end
			if (IsShiftKeyDown()) then
				UIParentLoadAddOn(self.addonName);
				UpdateTablet();
				return;
			end
			if (IsControlKeyDown()) then 
				return;
			end				
			if (not self.enabled) then
				EnableAddOn(self.addonName)
			else
				if (not (self.master == 1)) then
					DisableAddOn(self.addonName)
				end
			end
		else -- its a tree node
			if (IsAltKeyDown()) then
				for j in pairs(groupSets[self.addonName]) do		-- unhide all		
					if (type(groupSets[self.addonName][j]) == "string") then 
						-- this is an addon string													
						Broker_AddonControlChar.hiddenAddons[groupSets[self.addonName][j]] = nil;
					end
				end			
				return;
			end	
			if (IsControlKeyDown()) then -- collapse all nodes
				--self:Debug(#MCPFu_CharVars.expandedNodes,3);
				if (Broker_AddonControlChar.expandedNodes[self.addonName] == nil) then -- no nodes expanded			
					for i in pairs(groupSets) do
						--self:Debug(i,3);
						Broker_AddonControlChar.expandedNodes[i] = true;
					end
				else
					for i in pairs(Broker_AddonControlChar.expandedNodes) do
						--self:Debug(i,3);
						Broker_AddonControlChar.expandedNodes[i] = nil;
					end
				end
			elseif (IsShiftKeyDown()) then -- select / deselect all addons
				local enableOrDisable = 2;
				for j in pairs(groupSets[self.addonName]) do				
					if (type(groupSets[self.addonName][j]) == "string") then -- this is an addon string										
						if (not Broker_AddonControlChar.hiddenAddons[groupSets[self.addonName][j]]) then
							if (enableOrDisable == 2) then
								_, _, _, enableOrDisable = GetAddOnInfo(groupSets[self.addonName][j]);						
							end
							if (not (groupSets[self.addonName][j] == "Broker_AddonControl")) then
								if (not enableOrDisable) then							
									EnableAddOn(groupSets[self.addonName][j])
								else
									--self:Debug(groupSets[self.addonName][j],3);
									DisableAddOn(groupSets[self.addonName][j])
								end
							end
						end
					end
				end		
			else 
				if (Broker_AddonControlChar.expandedNodes[self.addonName]) then
					Broker_AddonControlChar.expandedNodes[self.addonName] = nil;
				else
					Broker_AddonControlChar.expandedNodes[self.addonName] = 1;
				end
			end
		end
	end
	UpdateTablet();	
end

local function Scroll(self, delta)
	if IsShiftKeyDown() then
		local width = Broker_AddonControl.GetConfigValue("maxDescWidth");
		width = width + delta * 20
		if (width < MIN_DESC_WIDTH) then 
			return 
		end
		Broker_AddonControl.SetConfigValue("maxDescWidth",width);		
		UpdateTablet()		
	elseif IsControlKeyDown() then
		if (not Broker_AddonControl.GetConfigValue("resizeTip")) then
			return;
		end
		local scale = Broker_AddonControl.GetConfigValue("scale");
		scale = scale + delta * 0.05
		Broker_AddonControl.SetConfigValue("scale",scale);		
		UpdateTablet()
	else
		slider:SetValue( sliderValue - delta * (IsModifierKeyDown() and 10 or 3) )
	end
end

buttons = setmetatable( { }, { __index = function( table, key )
	local button = CreateFrame( "Button", nil, f )
	table[key] = button
	button.index = key
	button:SetNormalFontObject(GameFontNormal)
	button:RegisterForClicks"AnyUp"
	button:SetScript( "OnEnter", Menu_OnEnter )
	button:SetScript( "OnLeave", Menu_OnLeave )

	button:EnableMouseWheel(true)
	button:SetScript( "OnMouseWheel", Scroll)

	button:SetHeight( BUTTON_HEIGHT )
	button.class = button:CreateTexture()
	button.class:SetWidth( ICON_SIZE ) button.class:SetHeight( ICON_SIZE )
	button.class:SetPoint( "LEFT", button, "LEFT" )
	
	--button.name = CreateFS( button, config.alignName )
	button.name = CreateFS( button, "LEFT" )
	button.name:SetPoint( "LEFT", button.class, "RIGHT", TEXT_OFFSET, 0 )
	button.desc = CreateFS( button, "CENTER", button.name )
	button.status  = CreateFS( button, "RIGHT", button.desc )
	--button.note = CreateFS( button, config.alignNote, button.zone, GAP, colors.note )
	--button.rank  = CreateFS( button, config.alignRank,  button.note, GAP, colors.rank )
	
	button:SetScript( "OnClick", OnAddonClick )
	return button
end } )

UpdateTablet = function()
	f:SetScale(Broker_AddonControl.GetConfigValue("scale"))
	local buttonNumber = 1;	
	local maxNameWidth, maxDescWidth, maxStatusWidth = 0,0,0
	local width
		
	if (toolTipModifier == "Shift") then				
		for i in ipairs(sortedSets) do sortedSets[i] = nil end
		for k in pairs(Broker_AddonControlConfig.sets) do table.insert(sortedSets, k) end
		table.sort(sortedSets)
		
		for i in pairs(sortedSets) do
			local button = buttons[buttonNumber];

			button.addonName = sortedSets[i];
			button.isNode = 0;
			button:SetWidth(1000)
			button.name:SetWidth(1000)
			button:SetAlpha(1);
			button:SetPoint( "TOPLEFT", f, "TOPLEFT", GAP, -GAP - ((buttonNumber-1)*(BUTTON_HEIGHT)) )			
			
			button.name:SetText( sortedSets[i] );
			width = button.name:GetStringWidth();
			if width > maxNameWidth then maxNameWidth = width end
			
			button.desc:SetText("");
			button.status:SetText("");
			button.name:SetTextColor(unpack(colors.altToolTip));
			button.class:SetTexture(nil);
			
			buttonNumber = buttonNumber+1;
		end	

		if (buttonNumber == 1) then 
			f:Hide();
			return;
		end				
	elseif (toolTipModifier == "Alt") then				
		local sorting = {};
		for i in pairs(Broker_AddonControlChar.hiddenAddons) do table.insert(sorting, i) end		
		table.sort(sorting)		
		
		for _,i in ipairs(sorting) do
			local button = buttons[buttonNumber];

			button.addonName = i;
			button.isNode = 0;
			button:SetWidth(1000)
			button.name:SetWidth(1000)
			button:SetAlpha(1);
			button:SetPoint( "TOPLEFT", f, "TOPLEFT", GAP, -GAP - ((buttonNumber-1)*(BUTTON_HEIGHT)) )			
			
			button.name:SetText( i );
			width = button.name:GetStringWidth();
			if width > maxNameWidth then maxNameWidth = width end
			
			button.desc:SetText("");
			button.status:SetText("");
			button.name:SetTextColor(unpack(colors.altToolTip));
			button.class:SetTexture(nil);
			
			buttonNumber = buttonNumber+1;
		end
		
		if (buttonNumber == 1) then 
			f:Hide();
			return;
		end		
	else	
		local sortGroup = {};		
		for i in pairs(groupSets) do		
			table.insert(sortGroup,i);
		end
		table.sort(sortGroup);
		table.sort(sortGroup, function(a, b)
			return strlower(a) < strlower(b)
		end )	
		for _,i in ipairs(sortGroup) do
			if (#groupSets[i] > 1) then -- there is more than 1 addon in this group / filter counts as 1
				local button = buttons[buttonNumber];

				button.addonName = i;
				button.isNode = 1;				
				button.name:SetTextColor(1,1,1)	
				button.desc:SetTextColor(unpack(colors.addonGroup))
				button.status:SetTextColor(unpack(colors.addonGroup))	
				button:SetWidth(2000)
				button.name:SetWidth(2000)
				button.desc:SetWidth(2000)
				button.status:SetWidth(2000)						
				button:SetAlpha(1);
				button:SetPoint( "TOPLEFT", f, "TOPLEFT", GAP, -GAP - ((buttonNumber-1)*(BUTTON_HEIGHT)) )			
			
				button.name:SetTextColor(unpack(colors.partEnabled));
				
				if (Broker_AddonControlChar.expandedNodes[button.addonName]) then
					button.class:SetTexture("Interface\\Buttons\\UI-MinusButton-Up");
				else
					button.class:SetTexture("Interface\\Buttons\\UI-PlusButton-Up");			
				end
				local tot, enab, load, hidden = Broker_AddonControl.GetAddonGroupStats(button.addonName);
				if (hidden > 0) then
					button.name:SetText( button.addonName.." ("..hidden.." "..L["hidden"]..")" );				
				else
					button.name:SetText( button.addonName );
				end
				width = button.name:GetStringWidth();
				if width > maxNameWidth then maxNameWidth = width end
				
				button.desc:SetText( L["enabled"].." "..enab.."/"..tot )
				width = button.desc:GetStringWidth();
				if width > maxDescWidth then maxDescWidth = width end																				
				
				button.status:SetText( L["loaded"].." "..load.."/"..tot )	
				width = button.status:GetStringWidth();
				if width > maxStatusWidth then maxStatusWidth = width end								
				
				if (enab+hidden == tot) then -- all enabled .. some hidden - show green
					button.name:SetTextColor(unpack(colors.allEnabled));
				elseif (enab == 0) then -- none enabled .. show red
					button.name:SetTextColor(unpack(colors.nonEnabled));
				end			
				if (tot ~= hidden) then --not all addons are hidden
					buttonNumber = buttonNumber +1; -- display it only if not all hidden
				end				
				button:Show();
			end
		
			if ( (#groupSets[i] == 1) or (Broker_AddonControlChar.expandedNodes[i]) ) then -- now show all addons in this group
				for j in pairs(groupSets[i]) do
					if ( (type(groupSets[i][j]) == "string") and (not Broker_AddonControlChar.hiddenAddons[groupSets[i][j]]) ) then 
					-- this is an addon string					
						local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(groupSets[i][j]);
						
						local button = buttons[buttonNumber];							
						button.addonName = name;
						button.isNode = 0;
						button.master = 0; -- make sure it cant be clicked off again
						button.desc:SetTextColor( 1,1,1 )
						button.name:SetTextColor( 1,1,1 )	
						button.status:SetTextColor( 1,1,1 )	
						button:SetWidth( 2000 )
						button.name:SetWidth(2000)
						button.desc:SetWidth(2000)
						button.status:SetWidth(2000)						
						button:SetAlpha(1);
						button:SetPoint( "TOPLEFT", f, "TOPLEFT", GAP, -GAP - ((buttonNumber-1)*(BUTTON_HEIGHT)) )			
						
						local nameTxt;
						if (Broker_AddonControlChar.expandedNodes[i]) then
							nameTxt = " - ";
						else
							nameTxt = "";
						end
						if (title) then						
							nameTxt = nameTxt..title;
						else
							nameTxt = nameTxt..name;
						end		

						button.name:SetText( nameTxt );													
						width = button.name:GetStringWidth();
						if width > maxNameWidth then maxNameWidth = width end					
						
						button.desc:SetText( notes );
						width = button.desc:GetStringWidth();
						if width > maxDescWidth then maxDescWidth = width end											
						
						local loaded = IsAddOnLoaded(name);
						local ondemand = IsAddOnLoadOnDemand(name);
						if (loadable) then -- addon is loadable
							button.name:SetTextColor(unpack(colors.activeAddon));					
						elseif (enabled and reason ~= "DEP_DISABLED") then
							button.name:SetTextColor(unpack(colors.nonEnabled));
						else
							button.name:SetTextColor(unpack(colors.disabledAddon));
						end						
			
						if (enabled) then
							button.class:SetTexture("Interface\\Buttons\\UI-CheckBox-Check");
							button.enabled = 1;
						else
							button.class:SetTexture(nil);
							button.enabled = nil;
						end		
						
						if (title == "Broker_AddonControl") then
							button.master = 1; -- make sure it cant be clicked off
							button.enabled = 1;
							button.class:SetTexture("Interface\\AddOns\\Broker_AddonControl\\icon");
						end
		
	--[[					if (self.db.profile.showSecurity) then
							if (security == "SECURE") then
								line['text3'] = L["SECURE"];				
							elseif (security == "INSECURE") then
								line['text3'] = L["INSECURE"];				
							elseif (security == "BANNED") then -- wtf?
								line['text3'] = L["BANNED"];				
							end
						end			
]]--
			
						if (reason) then			
							button.status:SetText( TEXT(getglobal("ADDON_"..reason)) )								
							if (reason == "DISABLED") or (reason == "WILL_NOT_LOAD") then
								button.class:SetTexture(nil);
								button.status:SetTextColor(unpack(colors.disabledAddon));
							end			
						elseif (loaded) then
							button.status:SetText( TEXT(ADDON_LOADED) )	
							button.status:SetTextColor(unpack(colors.loadedAddon));							
						elseif (ondemand) then
							button.status:SetText( L["Shift+Click to load"] )	
							button.status:SetTextColor(unpack(colors.clickToLoad));
						elseif (enabled) then
							button.status:SetText( L["ReloadUI to Load"] )	
							button.name:SetTextColor(unpack(colors.enabledAddon));					
						else
							button.status:SetText( "" )	
						end							

						width = button.status:GetStringWidth();
						if width > maxStatusWidth then maxStatusWidth = width end											
						
						button:Show();
						buttonNumber = buttonNumber +1; -- display it
					end
				end
			end 		
		end 	
	end
		
	MAX_ENTRIES = floor( (UIParent:GetHeight()/Broker_AddonControl.GetConfigValue("scale") -  GAP*2) / BUTTON_HEIGHT - ((Broker_AddonControl.GetConfigValue("showAddonTipInfo") and 9) or 5) / Broker_AddonControl.GetConfigValue("scale") )
	slider:SetHeight(BUTTON_HEIGHT*MAX_ENTRIES)
	hasSlider = (buttonNumber-1) > MAX_ENTRIES
	if hasSlider then
		slider:SetMinMaxValues( 0, (buttonNumber-1) - MAX_ENTRIES )
		slider:SetValue(sliderValue)
		slider:Show()
	else	slider:Hide() end
	nbEntries = math.min( MAX_ENTRIES, (buttonNumber-1) )

	UpdateScrollButtons(nbEntries)	
	
	if maxDescWidth > Broker_AddonControl.GetConfigValue("maxDescWidth") then
		maxDescWidth = Broker_AddonControl.GetConfigValue("maxDescWidth");
	end
	
	if (Broker_AddonControl.GetConfigValue("hideNotes")) then		
		width = maxNameWidth + maxStatusWidth + ICON_SIZE + TEXT_OFFSET;	
	else
		width = maxNameWidth + maxDescWidth + maxStatusWidth + ICON_SIZE + TEXT_OFFSET;	
	end	

	for i=1, #buttons do
		button = buttons[i]
		button:SetWidth( width)
		button.name:SetWidth(maxNameWidth)
		button.name:SetHeight(BUTTON_HEIGHT)		
		if (Broker_AddonControl.GetConfigValue("hideNotes")) then
			button.desc:SetText("");
			button.desc:Hide();
			button.desc:SetWidth(0)		
		else
			button.desc:SetWidth(maxDescWidth)		
			button.desc:SetHeight(BUTTON_HEIGHT)
		end				
		button.status:SetWidth(maxStatusWidth)
		button.status:SetHeight(BUTTON_HEIGHT)		
	end	
	
	f:SetHeight((nbEntries) *(BUTTON_HEIGHT) +10)
	f:SetWidth(width + 30)

	if hasSlider then slider:SetPoint("TOPRIGHT", buttons[1], "TOPRIGHT", 19 + TEXT_OFFSET, 0) end
	
	Broker_AddonControl.UpdateText();
end

local function AnchorTablet(frame)
	toolTipModifier = ""
	if (IsControlKeyDown()) then 
		toolTipModifier = "Ctrl"
		return;
	end
	if (IsAltKeyDown()) then 
		toolTipModifier = "Alt"
	end
	if (IsShiftKeyDown()) then 
		toolTipModifier = "Shift"		
	end	
	CloseDropDownMenus()	
	f.isTop, f.onBlock = select(2, frame:GetCenter()) > UIParent:GetHeight() / 2, true
	f:ClearAllPoints()
	f:SetPoint(f.isTop and "TOP" or "BOTTOM", frame, f.isTop and "BOTTOM" or "TOP")
	
	f:SetBackdropColor( unpack( colors.background ) )
	f:SetBackdropBorderColor( unpack( colors.border ) )
	f:SetScale(1)
	f:SetWidth(1)
	f:SetHeight(1)
	f:SetAlpha(1)		
	f:Show()	
	UpdateTablet()	
	UpdateBlockHints()
end

function Broker_AddonControl.VerifyHiddenAddonsDisabled()
	for i in pairs(Broker_AddonControlChar.hiddenAddons) do		
		DisableAddOn(i);
	end	
end

function Broker_AddonControl.UseSet(...)
	name = select(1, ...)
	setToUse = Broker_AddonControlConfig.sets[name]
	if type(setToUse) ~= "table" then return false end
	
	for k, _ in pairs(setToUse) do
		if (setToUse[k] == 1) then
			EnableAddOn(k);
		else
			DisableAddOn(k);
		end
	end
	
	reclaim(Broker_AddonControlChar.expandedNodes);
	Broker_AddonControlChar.expandedNodes = {};
	for i in pairs(setToUse.expandedNodes) do
		Broker_AddonControlChar.expandedNodes[i] = setToUse.expandedNodes[i];
	end		
	print(L["Loaded set: %s."]:format(name));
end

function Broker_AddonControl.CancelCreateSet()
	StaticPopup_Hide("BrokerAddonControlAddSet");
end

function Broker_AddonControl.CreateOrUpdateSet(name)
	if UnitAffectingCombat("player") or InCombatLockdown() then return end
	StaticPopup_Show("BrokerAddonControlAddSet");
end

function Broker_AddonControl.CreateSet(name)	
	Broker_AddonControl.AddSet(name);
	StaticPopup_Hide("BrokerAddonControlAddSet");
end

function Broker_AddonControl.HasSet(name)
	return Broker_AddonControlConfig.sets[name] ~= nil
end

function Broker_AddonControl.AddSet(name)
	local concatedName = string.gsub(name," ", "")
	if not name or name == "" or concatedName == "" then
		print(L["Please use a proper name for your set."])
		return
	end
	if Broker_AddonControl.HasSet(concatedName) then name = concatedName end

	local text = L["Added set: %s."]:format(name)
	--local event = "ClosetGnome_AddSet"
	if Broker_AddonControl.HasSet(name) then
		text = L["Updating set: %s."]:format(name)
		--event = "ClosetGnome_UpdateSet"
		reclaim(Broker_AddonControlConfig.sets[name]);
	end
	
	--add the set items now
	reclaim(Broker_AddonControlConfig.sets[name]);
	Broker_AddonControlConfig.sets[name] = {};
	local numAddons = GetNumAddOns();
	for i = 1, numAddons, 1 do		
		local aname, _, _, enabled = GetAddOnInfo(i);
		if (enabled) then
			Broker_AddonControlConfig.sets[name][aname] = 1;
		else
			Broker_AddonControlConfig.sets[name][aname] = 0;
		end
	end
	reclaim(Broker_AddonControlConfig.sets[name].expandedNodes);
	Broker_AddonControlConfig.sets[name].expandedNodes = {};
	for i in pairs(Broker_AddonControlChar.expandedNodes) do
		Broker_AddonControlConfig.sets[name].expandedNodes[i] = Broker_AddonControlChar.expandedNodes[i];
	end
	if not Broker_AddonControlConfig.sets[name] then return end
	print(text)
end

function Broker_AddonControl.DeleteSet(name, verified)
	if not verified and not IsShiftKeyDown() then
		queuedForDelete = name
		StaticPopup_Show("BrokerAddonControlDeleteSet", name)
		return
	end

	queuedForDelete = nil
	if Broker_AddonControlConfig.sets[name] then
		Broker_AddonControlConfig.sets[name] = reclaim(Broker_AddonControlConfig.sets[name])
		print(L["Deleted set: %s."]:format(name))
	end
end

function Broker_AddonControl.GetAddonGroupStats(groupName)
	local totalAddons =0;
	local enabledAddons = 0;
	local loadedAddons = 0;
	local hiddenAddons = 0;
	if (groupSets[groupName]) then
		for i in pairs(groupSets[groupName]) do
			if (type(groupSets[groupName][i]) == "string") then -- this is an addon string							
				local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(groupSets[groupName][i]);
				if (name) then
					totalAddons = totalAddons +1;
					if (enabled) then
						enabledAddons = enabledAddons +1;
					end
					if (IsAddOnLoaded(groupSets[groupName][i])) then
						loadedAddons = loadedAddons +1;
					end
					if (Broker_AddonControlChar.hiddenAddons[groupSets[groupName][i]]) then
						hiddenAddons = hiddenAddons +1;
					end
				end
			end
		end
	end
	return totalAddons, enabledAddons, loadedAddons, hiddenAddons;
end


function Broker_AddonControl.CreateAddonGroups()
	local numAddons = GetNumAddOns();
	groupSets = nil;
	groupSets = {};	
	
	
	if(not Broker_AddonControl.GetConfigValue("ignoreHardCodedGroups")) then	
		groupSets["Auctioneer"] = {};
		groupSets["Auctioneer"].filter = {};
		table.insert(groupSets["Auctioneer"].filter,"Auc");
		table.insert(groupSets["Auctioneer"].filter,"Enchantrix");
		table.insert(groupSets["Auctioneer"].filter,"SlideBar");
		table.insert(groupSets["Auctioneer"].filter,"!Swatter");
		table.insert(groupSets["Auctioneer"].filter,"BeanCounter");
		table.insert(groupSets["Auctioneer"].filter,"Stubby");
		table.insert(groupSets["Auctioneer"].filter,"Informant");
		
		groupSets["FuBar"] = {};
		groupSets["FuBar"].filter = {};
		table.insert(groupSets["FuBar"].filter,"FuBar");
		
		groupSets["Deadly Boss Mods"] = {};
		groupSets["Deadly Boss Mods"].filter = {};
		table.insert(groupSets["Deadly Boss Mods"].filter,"DBM-");	
		
		groupSets["cRange2"] = {};
		groupSets["cRange2"].filter = {};
		table.insert(groupSets["cRange2"].filter,"cRange2");		
		
		groupSets["ORA 2"] = {};
		groupSets["ORA 2"].filter = {};
		table.insert(groupSets["ORA 2"].filter,"oRA2");		
		
		groupSets["Bug Sack"] = {};
		groupSets["Bug Sack"].filter = {};
		table.insert(groupSets["Bug Sack"].filter,"BugSack");			
		table.insert(groupSets["Bug Sack"].filter,"!Bug");			
		
		groupSets["Satrina Buff Frame"] = {};
		groupSets["Satrina Buff Frame"].filter = {};
		table.insert(groupSets["Satrina Buff Frame"].filter,"Satrina");			
		table.insert(groupSets["Satrina Buff Frame"].filter,"SBF");				
		
		groupSets["Bulk Mail 2"] = {};
		groupSets["Bulk Mail 2"].filter = {};
		table.insert(groupSets["Bulk Mail 2"].filter,"BulkMail2");			
		
		groupSets["Libraries"] = {};
		groupSets["Libraries"].filter = {};
		table.insert(groupSets["Libraries"].filter,"Lib");			
		table.insert(groupSets["Libraries"].filter,"LIB");					
		
		groupSets["Shadowed Unit Frames"] = {};
		groupSets["Shadowed Unit Frames"].filter = {};
		table.insert(groupSets["Shadowed Unit Frames"].filter,"Shadowed");			
		
		groupSets["Data Broker Addons"] = {};
		groupSets["Data Broker Addons"].filter = {};
		table.insert(groupSets["Data Broker Addons"].filter,"Ara_Broker");			
		table.insert(groupSets["Data Broker Addons"].filter,"zz");					
		table.insert(groupSets["Data Broker Addons"].filter,"StatBlock");					
		table.insert(groupSets["Data Broker Addons"].filter,"Broker");					
		table.insert(groupSets["Data Broker Addons"].filter,"VertiBroker");					
		table.insert(groupSets["Data Broker Addons"].filter,"Fortress");					
		table.insert(groupSets["Data Broker Addons"].filter,"shPerformance");					
		table.insert(groupSets["Data Broker Addons"].filter,"Wintergrasp Timer");					
		table.insert(groupSets["Data Broker Addons"].filter,"WintergraspLDB");					
		table.insert(groupSets["Data Broker Addons"].filter,"Repair Broker");	
	end
		
		
	for i = 1, numAddons, 1 do		
		local name, title = GetAddOnInfo(i);
		local found = false;
		for i in pairs(groupSets) do
			for j in pairs(groupSets[i].filter) do
				local useTitle = false;
				if (title) then						
					--self:Debug(name.." - "..title,1);
					if (strfind(title,groupSets[i].filter[j]) == 1) then 
						useTitle = true;
					end
				end
				if (strfind(name,groupSets[i].filter[j]) == 1) or (useTitle == true) then
					--self:Debug(groupSets[i].filter[j],5);
					--self:Debug(name,5);
					table.insert(groupSets[i],name);
					found = true;
				end
			end
		end
		if (found == false) then
			groupSets[name] = {};
			groupSets[name].filter = {};
			table.insert(groupSets[name].filter,string.sub(name,1,Broker_AddonControl.GetConfigValue("maxMatchCharacters")));		
			table.insert(groupSets[name],name);
		end
	end	
	table.sort(groupSets)	
end

function Broker_AddonControl.PurgeHiddenList()	
	local purged  = 0;
	for h in pairs(Broker_AddonControlChar.hiddenAddons) do 		
		local found = false;
		print(h);
		for i in pairs(groupSets) do
			--if (#groupSets[i] > 1) then -- there is more than 1 addon in this group / filter counts as 1
				for j in pairs(groupSets[i]) do
					if (type(groupSets[i][j]) == "string") then
						if (h == groupSets[i][j]) then
							found = true;
							break;
						end
					end
				end
			--end
		end
		if (found == false) then
			--print(h.. " not found");
			Broker_AddonControlChar.hiddenAddons[h] = nil;
			purged = purged + 1;
		end
	end
	if (purged > 0) then
		print(L["Addons purged :"]..purged);
	end
end


function Broker_AddonControl.SlashCmdHandler(msg, editbox)
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("Broker_AddonControl")
	elseif (string.lower(msg) == "perf") then
		Broker_AddonControl.PrintPerformanceData()		
	else
		Broker_AddonControl.ShowHelp()
	end
end

function Broker_AddonControl.ShowHelp()
	print("Slash commands (/bac):")
	print(" /bac config: Open addon config menu (also found in Addon tab in Blizzard's Interface menu)")
end

function Broker_AddonControl.PrintPerformanceData()
	UpdateAddOnMemoryUsage()
	local mem = GetAddOnMemoryUsage("Broker_AddonControl")
	print("Broker_AddonControl is currently using " .. mem .. " kbytes of memory")
	collectgarbage(collect)
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage("Broker_AddonControl")
	print("Broker_AddonControl is currently using " .. mem .. " kbytes of memory after garbage collection")

end

function Broker_AddonControl.GetConfigValue(key)
	return Broker_AddonControlConfig[key]
end

function Broker_AddonControl.SetConfigValue(key, value)
	if (DefaultConfig[key] == value) then
		Broker_AddonControlConfig[key] = nil
	else 
		Broker_AddonControlConfig[key] = value
	end
end

function Broker_AddonControl.SetConfigToDefaults()
	print("Resetting config to defaults")
	Broker_AddonControlConfig = {}
	setmetatable(Broker_AddonControlConfig, {__index = DefaultConfig})
end

local pName = "Broker_AddonControl"
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local dataobj = ldb:GetDataObjectByName(pName) or ldb:NewDataObject(pName, {
	type = "data source",
	text = "AddonCTRL",
	label = "AC",
	icon = [[Interface\AddOns\Broker_AddonControl\icon]],
	OnEnter = function(self)
		AnchorTablet(self)
	end,	
	OnLeave = Block_OnLeave,
})


function dataobj.OnClick(self, button)
	--[[if (IsControlKeyDown()) then
		self:CreateAddonGroups();	
	end	]]
	if (IsShiftKeyDown()) then		
		Broker_AddonControl.CreateOrUpdateSet(name);
		return;
	end	
--	if (IsAltKeyDown()) then		reserved for reloadui with alt key
	--end
	if (IsControlKeyDown()) then
		return;
	end		
	if (button == "LeftButton") then
		if(Broker_AddonControl.GetConfigValue("altReloadUI") == false) or (IsAltKeyDown()) then
			ReloadUI();		
		end
	elseif (button == "RightButton") then
		InterfaceOptionsFrame_OpenToCategory("Broker_AddonControl")
	else
	end		
end

function Broker_AddonControl.UpdateText()
	dataobj.text = "AC";
	
	local numEnabled = 0;
	local numAddons = GetNumAddOns();
	
	for i = 1, numAddons, 1 do				
		enabled = select(4, GetAddOnInfo(i));						
		if (enabled) then
			numEnabled = numEnabled + 1;
		end	
	end
	dataobj.text = numEnabled.."/"..numAddons;
end
